/*
 * Copyright (c) Members of the EGEE Collaboration. 2004-2010.
 * See http://www.eu-egee.org/partners/ for details on the
 * copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.glite.apel.publisher;

import org.apache.log4j.Logger;
import org.glite.apel.core.ApelException;
import org.glite.apel.core.Log;
import org.glite.apel.core.db.ApelDB;
import org.glite.apel.core.db.MySQLImpl;
import java.util.ArrayList;
import org.glite.apel.core.config.parser.Config;
import java.util.Vector;

/**
 * Attempts to stitch together all the accounting data into the LcgRecord
 * schema. Formulated data is then stored locally within the Site's db and
 * then published via R-GMA.
 */
class AccountManager {

    /**
     * Reference to logging tool.
     */
    private static final Logger LOG =
            Logger.getLogger(AccountManager.class.getName());
    /**
     * No republishing is done, only new lcg records will be
     * published.
     */
    static final int REPUBLISH_NOTHING = 0;
    /**
     * Flag indicates missing records from last publish 
     * attempt should be republished (in addition to any new ones).
     */
    static final int REPUBLISH_MISSING = 1;
    /**
     * Flag indicates all lcg records should be republished.
     */
    static final int REPUBLISH_ALL = 2;
    /**
     * Flag indicates all lcg records in gap should be republished.
     */
    static final int REPUBLISH_GAP = 3;
    /**
     * The Gap interval.
     */
    private String m_start_date;
    private String m_end_date;
    /**
     * The disableJoin option. 
     */
    private boolean m_disableJoin;
    /**
     * Mysql db interface.
     */
    private ApelDB m_db;
    /**
     * Publishes any accounting records into rgma.
     */
    private AccountPublisher m_rgmaPub;
    /**
     * Defines which mode the join processor will operate (ie publish
     * missing, republish all, do nothing).
     */
    private int m_repubMode;
    /**
     * Site name publishing the accounting data.
     */
    private String m_siteName;
    /**
     * Database information
     */
    private String m_url;
    private String m_username;
    private String m_password;
    /**
     * Number of records to publish in each batch
     */
    private int m_limit;
    /**
     * Configuration file for the publisher
     */
    private Config m_config;
    /**
     * Location of the the public key file
     */
    private String m_publicKey;
    /**
     * Publishing UserDN?
     */
    private boolean m_publishUserName;

    public AccountManager(Config apelConfig) throws ApelException {

        m_config = apelConfig;

        m_url = m_config.getProperty(PublisherMap.PROP_DB_URL);
        m_username = m_config.getProperty(PublisherMap.PROP_DB_USERNAME);
        m_password = m_config.getProperty(PublisherMap.PROP_DB_PASSWORD);
        m_siteName = m_config.getProperty(PublisherMap.PROP_SITE);
        String limit = m_config.getProperty(PublisherMap.PROP_LIMIT);
        try {
            m_limit = Integer.parseInt(limit);
        } catch (Exception e) {
            m_limit = 300000;
            Log.info(LOG, "Limit not set in configuration file, using default: 300000");
        }

        // Checking what mode of republishing is set
        setRepublishMode();

        // Setting the GAP data
        String rs = m_config.getOptionValue(PublisherMap.OPT_REPUBLISH_RECORDSTART);
        String re = m_config.getOptionValue(PublisherMap.OPT_REPUBLISH_RECORDEND);
        m_start_date = rs;
        m_end_date = re;

        m_publishUserName = m_config.isOptionSet(PublisherMap.OPT_PUBLISH_GLOBAL_USER);
        m_disableJoin = m_config.isOptionSet(PublisherMap.OPT_REPUBLISH_NOJOIN);
        // If using public key file, set location
        if (m_config.getOptionValue(PublisherMap.OPT_PUBLIC_KEY_LOCATION) != null) {
            m_publicKey = (m_config.getOptionValue(PublisherMap.OPT_PUBLIC_KEY_LOCATION));
        }

        m_db = new MySQLImpl(m_url, m_username, m_password, m_siteName);
        setUpVOInfo();

    }

    /**
     * Carry out the processing to update the LcgRecords table.
     */
    public void run() throws ApelException {

        Log.info(LOG, "**** Running the APEL publisher in " + getRepublishModeMessage() + " ****");

        // Checking there is SpecRecords, but only if joining records
        if (!m_disableJoin) {
            Log.info(LOG, "Checking valid CPU spec data exists");
        }

        if ((!m_disableJoin) && m_db.count(ApelDB.SPEC_TABLE_NAME) == 0) {
            if (m_repubMode == REPUBLISH_ALL) {
                Log.warn(LOG, LogMsgs.NO_SPEC_DEFINED_WITH_REPROCESS);
            } else {
                Log.die(LOG, LogMsgs.NO_SPEC_DEFINED_IN_DB);
            }
        } else {
            Log.info(LOG, "CPU spec values found");
        }

        // Checking for local and server count only for publish all and missing
        if (m_repubMode == REPUBLISH_MISSING || m_repubMode == REPUBLISH_ALL) {
            chkArchivedTuples();
        }

        // If republish is gap, set dates, publish records, update timestamps and exit
        if (m_repubMode == REPUBLISH_GAP) {
            publishGapRecords();
            return;
        }

        // Join records
        if (!m_disableJoin){
            joinRecords();
        }

        // If republish is nothing, there is nothing else to do
        if (m_repubMode == REPUBLISH_NOTHING) {
            return;
        }

        // Publish records in LcgRecords table
        publishRecords();

        publishSync();

        m_rgmaPub.close();

    }

    /**
     * Checks the GOC contains the same number of tuples that were
     * published when the APEL was last run; republishes data if the GOC
     * is missing any records. 
     * 
     * Recovery mechanism is based on a stored time stamp which records the
     * last successful moment when all tuples were archived. This is
     * used to select tuples from the local/server LcgRecords table. A select
     * query is composed to count all records that match or exceed the
     * given time stamp. If the count matches, both the local and server view
     * are synchronised. If the server view contains fewer tuples, all
     * accounting records which match or exceed the time stamp are resent.
     */
    private void chkArchivedTuples() throws ApelException {

        Log.info(LOG, " ====================================");
        Log.info(LOG, "    Synchronisation data check       ");
        Log.info(LOG, " ====================================");

        if (!m_db.containsRepublishTimeStamp()) {
            Log.warn(LOG, LogMsgs.CANNOT_FIND_LAST_START_TIME);
            m_db.updateRepublishTimeStamp();
            return;
        }

        // Get the RepublishInfo timestamp
        String date = m_db.getRepublishDate();
        String time = m_db.getRepublishTime();

        Log.info(LOG, "Finding all records in local database since the "
                + "last successful publish timestamp : " + date + " " + time);

        int localCount = m_db.getNumAccountingRecords(date, time, m_siteName);

        if (localCount == 0) {
            Log.info(LOG, "No records found");
        } else {
            Log.info(LOG, "Record/s found in local database: " + localCount);
            AccountChecker con = new AccountChecker(m_config);
            int archivedCount = con.countArchivedRecords(date, time, m_siteName);
            if (archivedCount == -1) {
                Log.info(LOG, "Server Not Responding: Please inform APEL-SUPPORT@JISCMAIL.AC.UK");
            } else {
                Log.info(LOG, "Server Record Count: Record/s found site " + m_siteName + ": " + archivedCount);
                if (archivedCount == localCount) {
                    Log.info(LOG, "APEL Server and local db match-up, updating the RepublishInfo table");
                    m_db.updateRepublishTimeStamp();
                } else if (archivedCount < localCount) {
                    Log.info(LOG, LogMsgs.DETECTED_MISSING_RECORDS + date + " " + time);

                    if (m_rgmaPub == null) {

                        // Creating the publisher
                        m_rgmaPub = new AccountPublisher(m_config, m_publishUserName);
                        m_rgmaPub.setPublicKeyLocation(m_publicKey);
                    }
                    this.publishRecords();

                    // Check if the syncronization was successful
                    Log.info(LOG, "Checking the record counts for syncronisation ");
                    try {
                        Thread.sleep(2 * 60 * 1000);
                    } catch (InterruptedException ie) {
                    }
                    int new_archivedCount = con.countArchivedRecords(date, time, m_siteName);
                    Log.info(LOG, "Server Record Count: " + new_archivedCount);
                    if (new_archivedCount == localCount) {
                        Log.info(LOG, "Local database and GOC managed to syncronise, updating RepublishInfo");
                        m_db.updateRepublishTimeStamp();
                    } else {
                        Log.info(LOG, "Record Counts don't matchup, will not update RepublishInfo");
                    }
                } else {
                    Log.info(LOG, LogMsgs.UNABLE_TO_SYNCHRONIZE);
                }
            }
        }
        Log.info(LOG, " ====================================");
        Log.info(LOG, " Completed Synchronisation data check");
        Log.info(LOG, " ====================================");

    }

    /**
     * Republishes all existing accounting records and any new ones too for the CEs
     */
    private void joinRecords() throws ApelException {

        boolean ignoreProcessFlag = false;
        if (m_repubMode == REPUBLISH_ALL) {
            ignoreProcessFlag = true;
        }

        //
        // JOINS ARE PRIORITISED: Accounting_Log/LRMS; followed by GK/messages/LRMS
        // Why? Because, sites may process GK files when patch 898 is active
        // As the RecordIdententities derived from the accouting file and the GK
        // are different, there is the possibility that you could get double counting.
        // To prevent this, we always try to Join the accounting_log data first. This
        // sets LRMS data flags preventing and GK data using the same LRMSIDs to be joined.
        //

        // Accounting Log Data based on BlahdProcessor
        Log.info(LOG, "Building account records via the new Accounting Log File");
        Log.info(LOG, "NB: Record Counts may be zero if Patch #898 is not active on this CE");

        int totalCount = 0;
        int count = 0;
        int offset = 0;
        do {
            ApelDB db = new MySQLImpl(m_url, m_username, m_password, m_siteName);
            ArrayList ariGlite = db.generateAccountRecordsGliteCE(ignoreProcessFlag, m_limit, offset);
            Log.info(LOG, "Generating " + ariGlite.size() + " records");
            count = ariGlite.size();
            totalCount = totalCount + count;
            offset = m_limit + offset;
            ariGlite.clear();
            db.close();
        } while (count >= m_limit);
        Log.info(LOG, "Number of Joined accounting records: " + totalCount);


        // LCG Flavour CE based on GkProcessor
        Log.info(LOG, "Building account records via GK Logs");
        Log.info(LOG, "NB: Record Counts may be zero if Patch #898 is active on this CE");
        totalCount = 0;
        count = 0;
        offset = 0;
        do {
            ApelDB db = new MySQLImpl(m_url, m_username, m_password, m_siteName);
            ArrayList ari = db.generateAccountRecords(ignoreProcessFlag, m_limit, offset);
            Log.info(LOG, "Generating " + ari.size() + " records");
            count = ari.size();
            totalCount = totalCount + count;
            offset = m_limit + offset;
            ari.clear();
            db.close();
        } while (count >= m_limit);
        Log.info(LOG, "Number of Joined accounting records: " + totalCount);
        Log.info(LOG, "Build complete");

    }

    private void publishRecords() throws ApelException {

        int offset = 0;
        int totalCount = 0;
        int count = 0;

        String date = m_db.getRepublishDate();
        String time = m_db.getRepublishTime();

        if (m_rgmaPub == null) {

            // Creating the publisher
            m_rgmaPub = new AccountPublisher(m_config, m_publishUserName);
            m_rgmaPub.setPublicKeyLocation(m_publicKey);
        }
        do {
            ArrayList ari = new ArrayList();
            ApelDB db = new MySQLImpl(m_url, m_username, m_password, m_siteName);
            if (m_repubMode == REPUBLISH_MISSING){
                ari = db.getAccountingRecords(date, time, this.m_siteName, m_limit, offset);
            }
            else {
                ari = db.getAllAccountingRecords(m_limit, offset);
            }
            m_rgmaPub.publish(ari);
            offset = offset + m_limit;
            count = ari.size();
            Log.info(LOG, "Publishing " + count + " records...");
            totalCount = totalCount + count;
            ari.clear();
            db.close();
        } while (count >= m_limit);
        Log.info(LOG, "Published " + totalCount + " records to the APEL server. ");
	Log.info(LOG, "If filtering by VO, the number of records joined might differ from the number of records published");
    }

        private void publishSync() throws ApelException {

        if (m_rgmaPub == null) {

            // Creating the publisher
            m_rgmaPub = new AccountPublisher(m_config, m_publishUserName);
            m_rgmaPub.setPublicKeyLocation(m_publicKey);
        }
        // Publish sync summary
        Log.info(LOG, " ====================================");
        Log.info(LOG, "      Publishing Summary Data        ");
        Log.info(LOG, " ====================================");
        Log.info(LOG, "Data will be written to Sync Table: " + ApelDB.LCG_RECORDS_SYNC_TABLE_NAME);

        m_rgmaPub.publish(m_db.getAccountingSummary(m_siteName));


    }

    /**
     * Publishes all existing accounting records in Gap
     * Returns the number of published accounting records
     */
    private void publishGapRecords() throws ApelException {

        // Creating the publisher
        m_rgmaPub = new AccountPublisher(m_config, m_publishUserName);
        m_rgmaPub.setPublicKeyLocation(m_publicKey);

        int offset = 0;
        int totalCount = 0;
        int count = 0;
        do {
            ArrayList ari = new ArrayList();
            ApelDB db = new MySQLImpl(m_url, m_username, m_password, m_siteName);
            ari = db.getAccountingRecordsInGap(m_start_date, m_end_date, m_limit, offset);
            count = ari.size();
            Log.info(LOG, "Publishing " + count + " records...");
            m_rgmaPub.publish(ari);

            // After the records are processed, we should update their MeasurmentDate and MeasurmentTime
            // so that when GOC counts and local counts are made, these records are taken into account, and
            // the RepublishInfo table updated.
            // Otherwise, GOC count will ALWAYS exceed local count and the RepublishInfo table will not
            // be updated.
            Log.info(LOG, "Updating MeasurementDate and MeasurementTime timestamp of Gap Data");
            db.updateAccountingTimeStamp(ari);
            offset = offset + m_limit;
            totalCount = totalCount + count;
            ari.clear();
            db.close();
        } while (count >= m_limit);
        Log.info(LOG, "Total records published to Server with GAP Publisher:" + totalCount);

    }

    private String getRepublishModeMessage() {
        if (m_repubMode == REPUBLISH_NOTHING) {
            return "Republish Nothing Mode";
        }
        if (m_repubMode == REPUBLISH_GAP) {
            return "Republish Gap Mode";
        }
        if (m_repubMode == REPUBLISH_MISSING) {
            return "Republish Missing Mode";
        }
        if (m_repubMode == REPUBLISH_ALL) {
            return "Republish All Mode";
        }
        return "";
    }

    private void setRepublishMode() throws ApelException {
        if (m_config.isPropertySet(PublisherMap.PROP_REPUBLISH)) {
            String repubMode =
                    m_config.getProperty(PublisherMap.PROP_REPUBLISH);
            if (repubMode.equals(PublisherMap.REPUBLISH_ALL)) {
                m_repubMode = AccountManager.REPUBLISH_ALL;
            } else if (repubMode.equals(PublisherMap.REPUBLISH_MISSING)) {
                m_repubMode = AccountManager.REPUBLISH_MISSING;
            } else if (repubMode.equals(PublisherMap.REPUBLISH_NOTHING)) {
                m_repubMode = AccountManager.REPUBLISH_NOTHING;
                /** DaveKant --BEGIN -- */
            } else if (repubMode.equals(PublisherMap.REPUBLISH_GAP)) {
                m_repubMode = AccountManager.REPUBLISH_GAP;
                /** DaveKant --END -- */
            } else {
                // throw Excpetion
                throw new ApelException("Unknown value '" + repubMode
                        + "' defined in config file, value must be one of "
                        + "[" + PublisherMap.REPUBLISH_ALL + ", "
                        + PublisherMap.REPUBLISH_MISSING + ", "
                        + PublisherMap.REPUBLISH_NOTHING + "] check user "
                        + "documentation for further help");
            }
        } else {
            // If no republish mode is defined, set a default
            m_repubMode = AccountManager.REPUBLISH_MISSING;
        }
    }

    private void setUpVOInfo(){
        // Get include setting (all | include | exclude)
        String include_type = m_config.getOptionValue(PublisherMap.OPT_VOINCLUDE_INCLUDE);
        Vector<String> VOList = m_config.getPropertyValues(PublisherMap.PROP_VO);
        Vector<String> VOGroup = m_config.getPropertyValues(PublisherMap.PROP_VOGROUP);

        // If there is no VOInclude section in the configuration file,
        // Publish all VOs (include="all")
        if (include_type == null){
            include_type = "all";
            Log.info(LOG, "VO Filtering not set: Publishing all VOs");
        }

        // Create an ArrayList with the list of VOs to include/exclude
        ArrayList VOList_array = new ArrayList<String>();
        for (String volist : VOList) {
            VOList_array.add(volist);
        }

        // Create an ArrayList from the VOGroupings (better than using the vector)
        ArrayList VOGrouping = new ArrayList<String>();
        for (String vogroup: VOGroup){
            VOGrouping.add(vogroup);
        }

        // Boolean to define if we are including or excluding
        boolean include = false;

        // If include=all, we ignore whatever is in the VOlist variables
        if (include_type.equals("all")){
            include = true;
            VOList_array.clear();
        } else if (include_type.equals("include")) {
            include = true;
        } else if (include_type.equals("exclude")) {
            include = false;
        } else {
            Log.die(LOG, "Wrong option in VOInclude include. Must be all | include | exclude");
        }

        MySQLImpl.setVOFilter(VOList_array, include);
        MySQLImpl.setVOMapping(VOGrouping);
    }
}
