/*
 * Copyright (c) Members of the EGEE Collaboration. 2004-2010.
 * See http://www.eu-egee.org/partners/ for details on the
 * copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.glite.apel.publisher;

import java.util.ArrayList;


import javax.jms.DeliveryMode;
import javax.jms.JMSException;
import javax.jms.MessageProducer;
import javax.jms.ObjectMessage;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicSession;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import org.apache.log4j.Logger;
import org.glite.apel.core.ApelException;
import org.glite.apel.core.Log;
import org.glite.apel.core.UTCTime;
import org.glite.apel.core.db.ApelDB;
import org.glite.apel.core.log.type.AccountRecord;
import org.glite.apel.core.log.type.AccountSummary;
import org.glite.apel.core.log.type.AccountSummaryIterator;
import org.glite.apel.core.SecurityUtils;
import org.glite.apel.core.config.parser.Config;

/**
 * Class used to publish accounting data using an ActiveMQ Producer.
 */
class AccountPublisher {

    /**
     * Using Logj4.
     */
    private Logger m_log;
    /**
     * A list of pending insert statements.
     */
    private ArrayList<String> m_inserts;
    /**
     * Current number of batched insert statements.
     */
    private int m_numBatchedInserts;
    /**
     * Flag used to publish the globalUserName into ActiveMQ Broker.
     */
    private boolean m_publishUserName;
    /**
     * Using an ActiveMQ MessageProducer.
     */
    private MessageProducer m_producer;
    /**
     * A utility class which handles encryption.
     */
    private SecurityUtils m_securityUtils;
    /**
     * Location of the public key file
     */
    private String m_publicKeyLocation;
    /**
     * A utility class which formats the date/time into UTC.
     */
    private UTCTime m_utcTime;
    private TopicConnection m_topicConnection;
    private TopicSession m_topicSession;
    private Topic m_topic;
    private Config m_config;
    private static int MaxInsertBatch;

    AccountPublisher(Config apelConfig, boolean publishUserName) throws ApelException {
        m_log = Logger.getLogger(getClass().getName());
        m_inserts = new ArrayList<String>();
        m_utcTime = new UTCTime();
        m_publishUserName = publishUserName;
        m_config = apelConfig;

        try {
            MaxInsertBatch = Integer.parseInt(m_config.getProperty(PublisherMap.PROP_MAX_INSERT_BATCH));
        } catch (Exception e) {
            Log.die(m_log, e.getMessage());
        }

        try {
            createActiveMQProducer();
        } catch (Exception e) {
            Log.die(m_log, e.getMessage());
        }
    }

    /**
     * Create the ActiveMQ Producer.
     */
    private void createActiveMQProducer() throws ApelException {
        try {
            javax.naming.Context ctx = new InitialContext();

            //lookup the connection factory
            TopicConnectionFactory topicConnectionFactory = (TopicConnectionFactory) ctx.lookup("TopicConnectionFactory");

            // lookup an existing topic
            m_topic = (Topic) ctx.lookup("grid.accounting.apel.cpujobs.individualjobs");

            // create a new TopicConnection for pub messaging
            m_topicConnection = topicConnectionFactory.createTopicConnection();

            // create a new TopicSession for the client
            m_topicSession = m_topicConnection.createTopicSession(false, TopicSession.AUTO_ACKNOWLEDGE);

            m_producer = m_topicSession.createProducer(m_topic);
            m_producer.setDeliveryMode(DeliveryMode.NON_PERSISTENT);
            m_topicConnection.start();
        } catch (NamingException e) {
            Log.warn(m_log, e.getMessage());
        } catch (JMSException e) {
            throw new ApelException(e);
        }
    }

    /**
     * Adds the complete result set in one go. The result set must match
     * the same table format as the LcgRecords.
     */
    public void publish(ArrayList<AccountRecord> it) throws ApelException {

        m_utcTime.updateDateTime();
        Log.info(m_log, "Publishing data into ActiveMQ Broker");
        if (m_publishUserName) {
            Log.info(m_log, "  All UserDNs will be encrypted");
            m_securityUtils = SecurityUtils.getInstance();
            m_securityUtils.setPublicKeyLocation(m_publicKeyLocation);
        }
        for (int i = 0; i < it.size(); i++) {
            AccountRecord record = it.get(i);
            // set GlobalUserName to NULL if publishUserName is disabled
            String globalUserName;
            if (m_publishUserName) {
                globalUserName = m_securityUtils.EncryptDataToBase64(record.getLcgUserId());
            } else {
                globalUserName = ApelDB.NULL;
            }

            String globalJobId = record.getLcgJobId();
            if (globalJobId == null) {
                globalJobId = ApelDB.NULL;
            }

            String insert = "REPLACE INTO "
                    + ApelDB.LCG_RECORDS_TABLE_NAME
                    + "(RecordIdentity, ExecutingSite, LocalJobID, "
                    + " LCGJobID, LocalUserID, LCGUserID, LCGUserVO, "
                    + " ElapsedTime, BaseCpuTime, ElapsedTimeSeconds, "
                    + " BaseCpuTimeSeconds, StartTime, StopTime, "
                    + " StartTimeUTC, StopTimeUTC, "
                    + " StartTimeEpoch, StopTimeEpoch, ExecutingCE, "
                    + " MemoryReal, MemoryVirtual, SpecInt2000, "
                    + " SpecFloat2000, EventDate, EventTime, "
                    + " MeasurementDate, MeasurementTime)"
                    + " VALUES ('" + record.getRecordId() + "', '"
                    + record.getExecutingSite() + "', '"
                    + record.getLocalJobId() + "', '"
                    + globalJobId + "', '"
                    + record.getLocalUserId() + "', '"
                    + globalUserName + "', '"
                    + record.getLocalUserVO() + "', '"
                    + record.getElapsedTime() + "', '"
                    + record.getBaseCPUTime() + "', "
                    + record.getElapsedTimeSeconds() + ", "
                    + record.getBaseCPUTimeSeconds() + ", '"
                    + record.getStartTime() + "', '"
                    + record.getStopTime() + "', '"
                    + record.getStartTimeUtc() + "', '"
                    + record.getStopTimeUtc() + "', "
                    + record.getStartTimeEpoch() + ", "
                    + record.getStopTimeEpoch() + ", '"
                    + record.getExecutingCE() + "', "
                    + record.getMemoryReal() + ", "
                    + record.getMemoryVirtual() + ", "
                    + record.getSpecInt2000() + ", "
                    + record.getSpecFloat2000() + ", '"
                    + record.getEventDate() + "', '"
                    + record.getEventTime() + "', '"
                    + m_utcTime.getDate() + "', '"
                    + m_utcTime.getTime() + "')";

            m_log.debug(insert);
            m_inserts.add(insert);
            m_numBatchedInserts++;

            if (m_inserts.size() > MaxInsertBatch) {
                executeBatch();
                m_utcTime.updateDateTime();
            }

        }
        executeBatch();

    }

    /**
     * Adds the complete result set in one go. The result set must match
     * the same table format as the LcgRecordsSync_v01.
     */
    public void publish(AccountSummaryIterator it) throws ApelException {
        Log.info(m_log, "Publishing summary data into ActiveMQ Broker");
        m_utcTime.updateDateTime();

        while (it.hasNext()) {
            AccountSummary record = it.next();

            String insert = "REPLACE INTO " + ApelDB.LCG_RECORDS_SYNC_TABLE_NAME
                    + "(RecordIdentity, ExecutingSite, Njobs, Ndays, RecordStart, "
                    + "RecordEnd, MeasurementDate, MeasurementTime)"
                    + " VALUES ('"
                    + record.getRecordId() + "', '"
                    + record.getExecutingSite() + "', "
                    + record.getNjobs() + ", "
                    + record.getNdays() + ", "
                    + //Warning: These will be treated as INTEGERS if they are not emcapsulated in single quotes!
                    //record.getElapsedTimeSeconds() + ", " +
                    //record.getBaseCpuTimeSeconds() + ", '" +
                    "'" + record.getRecordStart() + "', "
                    + "'" + record.getRecordEnd() + "', "
                    + "'" + m_utcTime.getDate() + "', "
                    + "'" + m_utcTime.getTime() + "')";

            m_log.debug(insert);
            m_inserts.add(insert);
            m_numBatchedInserts++;

            if (m_inserts.size() > MaxInsertBatch) {
                executeBatch();
                m_utcTime.updateDateTime();
            }
        }
        executeBatch();

    }

    /**
     * Publishes the buffer if enough tuples exist, if
     */
    private void executeBatch() throws ApelException {

        if (m_inserts.size() > 0) {
            boolean published = false;
            do {
                try {
                    m_numBatchedInserts = 0;
                    ObjectMessage recordMessage = m_topicSession.createObjectMessage(m_inserts);
                    m_producer.send(recordMessage);
                    published = true;
                    m_inserts.clear();
                } catch (Exception e) {
                    handleException(e);
                }
            } while (!published);

        }
    }

    private void handleException(Exception e) throws ApelException {
        Log.info(m_log, "Exception thrown but not handled. Please submit a GGUS ticket");
        throw new ApelException(e);
    }

    /**
     * Tidies up any open resources (closes the StreamProducer etc).
     */
    public void close() {
        try {
            m_producer.close();
            m_topicSession.close();
            m_topicConnection.close();
        } catch (Exception e) {
            m_log.debug("encountered error while closing producer: " + e);
        }
    }

    public void setPublicKeyLocation(String publicKey) {
        m_publicKeyLocation = publicKey;
    }
}
