/*
 * Copyright (c) Members of the EGEE Collaboration. 2004-2010.
 * See http://www.eu-egee.org/partners/ for details on the
 * copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.glite.apel.publisher;

import java.io.FileNotFoundException;
import org.apache.log4j.Logger;
import org.glite.apel.core.ApelException;
import org.glite.apel.core.processor.DeleteProcessor;
import org.glite.apel.core.processor.DBProcessor;
import org.glite.apel.core.Log;
import org.glite.apel.core.config.map.CLMap;
import org.glite.apel.core.config.parser.CLParser;
import org.glite.apel.core.config.parser.Config;
import org.glite.apel.core.config.parser.ConfigurationException;
import org.glite.apel.core.config.parser.XMLParser;
import java.util.Vector;
import org.apache.log4j.PropertyConfigurator;

/**
 * The main entry point for starting the apel accounting application.
 */
public class ApelPublisher {

    /**
     * Using log4j logging.
     */
    private static final Logger LOG =
            Logger.getLogger(ApelPublisher.class.getName());
    /**
     * The script name used.
     */
    private static final String PROG_NAME = "apel-publisher";
    /**
     * Single reference that points to all the properties/configs
     * for this application.
     */
    private Config m_config;

    private String configFileName;

    /**
     * Sets-up the user configs for the apel application.
     */
    public ApelPublisher(String[] args) {
        String log4jFile = System.getProperty("log4j.configuration");
        log4jFile = log4jFile.replace("file:", "");
        PropertyConfigurator.configure(log4jFile);
        Log.setProgramName(PROG_NAME);
        setUp(args);
    }

    /**
     * Kick starts the apel application.
     */
    private void run() {

        String url = m_config.getProperty(PublisherMap.PROP_DB_URL);
        String user = m_config.getProperty(PublisherMap.PROP_DB_USERNAME);
        String password = m_config.getProperty(PublisherMap.PROP_DB_PASSWORD);
        String siteName = m_config.getProperty(PublisherMap.PROP_SITE);

        try {
            Log.info(LOG, "------ Starting the APEL Publisher ------");
            if (m_config.isOptionSet(PublisherMap.OPT_DB_DELETE_PROCESSOR)) {
                DeleteProcessor deleteProc = new DeleteProcessor(url, user, password, siteName);
                boolean deleteAll = m_config.isOptionSet(PublisherMap.OPT_CLEAN_ALL_FROM_TABLES);
                deleteProc.run(deleteAll);
            }
            if (m_config.isOptionSet(PublisherMap.OPT_DB_PROCESSOR)) {
                DBProcessor dbProc = new DBProcessor(url, user, password, siteName);
                boolean inspectTables = m_config.isOptionSet(PublisherMap.OPT_INSPECT_TABLES);
                dbProc.run(inspectTables);
            }
            if (m_config.isOptionSet(PublisherMap.OPT_JOIN_PROCESSOR)) {
                runJoinProcessor();
            }
            Log.info(LOG, "------ APEL Publisher finished ------");
            System.exit(0);
        } catch (ApelException ae) {
            Log.die(LOG, ae);
        } catch (Exception e) {
            Log.die(LOG, "Internal program error - please report bug", e);
        }
    }

    /**
     * Attempts to run the join processor (to generate new accounting
     * records and to publish them into rgma).
     */
    private void runJoinProcessor() throws ApelException {
        AccountManager joinProc = new AccountManager(m_config);
        joinProc.run();
    }

    /**
     * Intialises input parameter defined on the command line
     * and then attempts to read-in configs from a property file.
     * The location of the property file is defined on the command line
     */
    private void setUp(String[] args) {

        try {
            if (args.length == 0) {
                help();
            }
            CLParser clParser = new CLParser(args);
            Config clConfig = clParser.parse(new CLMap());
            processStandardOptions(clConfig);
            configFileName =
                    clConfig.getOptionalProperty(CLMap.PROP_CONFIG_FILE);
            if (configFileName == null) {
                Log.die(LOG, "No config file defined, use -h or "
                        + "--help for more info");
            }
            LOG.debug("using config file: " + configFileName);
            XMLParser xmlParser = new XMLParser(configFileName);
            m_config = xmlParser.parse(new PublisherMap());
        } catch (ConfigurationException e) {
            Log.die(LOG, e.getMessage());
        } catch (FileNotFoundException e) {
            Log.die(LOG, "File not found: " + configFileName);
        }
        if (m_config.isOptionSet(PublisherMap.OPT_ENABLE_LOGGING)) {
            Log.setQuiet(false);
        }
        Vector hideProperties = new Vector();
        hideProperties.add(PublisherMap.PROP_DB_PASSWORD);
        Log.info(LOG, "Read-in configuration: "
                + m_config.toString(hideProperties));
    }

    /**
     * Checks and processes 'help', 'version' and 'quiet' options.
     */
    private void processStandardOptions(Config config) {
        if (config.isOptionSet(CLMap.OPT_HELP)) {
            help();
        }
        if (config.isOptionSet(CLMap.OPT_VERSION)) {
            showVersion();
        }
    }

    /**
     * Displays help information and terminate.
     */
    private void help() {
        System.out.println();
        System.out.print("The " + PROG_NAME + " is an accounting tool that ");
        System.out.print("attempts to collate data produced from parsed ");
        System.out.print("event logs and Globus gate keeper logs ");
        System.out.print("into accounting records. ");
        System.out.print("These records are then published using R-GMA.");
        System.out.print("Please refer to user guide for full installation ");
        System.out.println("and setup instructions. ");
        System.out.println();
        System.out.println("Usage: " + PROG_NAME + " [option]...");
        System.out.println();
        System.out.println("\t-f, --config[=filename]");
        System.out.println();
        System.out.println("\tthe config file to be used");
        System.out.println();
        System.out.println("\t-h, --help");
        System.out.println();
        System.out.println("\tdisplay this help and exit");
        System.out.println();
        System.out.println("\t-v, --version");
        System.out.println();
        System.out.println("\toutput version information and exit");
        System.out.println();
        System.out.println("Examples:");
        System.out.println();
        System.out.println("\t" + PROG_NAME + " -f /home/zucchini/config.xml");
        System.out.println();
        System.out.println("Report bugs to Savannah at "
                + "https://savannah.cern.ch/bugs/?group=jra1mdw");
        System.out.println();
        System.exit(0);
    }

    /**
     * Display program version number and exits.
     */
    private void showVersion() {
        // CVS will replace this string with the correct information
        String cvsTag = "$Name: glite-apel-publisher_R_2_0_14_0 $";
        if (cvsTag.matches(".Name:  .")) {
            cvsTag = "Development";
        } else {
            cvsTag = cvsTag.replaceFirst(".Name: ", "");
            if (cvsTag.length() >= 2) {
                cvsTag = cvsTag.substring(0, cvsTag.length() - 2);
            }
        }
        System.out.println();
        System.out.println(PROG_NAME + " version " + cvsTag);
        System.out.println("Copyright (c) Members of the EGEE Collaboration. " + "2004. See http://eu-egee.org/partners/ for details on the "
                + "copyright holders."
                + "For license conditions see For license conditions see the "
                + "license file or http://eu-egee.org/license.html");
        System.out.println();
        System.exit(0);
    }

    /**
     * Create an APEL object and call its start method. This is the program
     * entry point for processing of data. The archiver program has a
     * different entry point.
     */
    public static void main(String[] args) {
        ApelPublisher apel = new ApelPublisher(args);
        apel.run();
    }
}
