/*
 * Copyright (c) Members of the EGEE Collaboration. 2004-2010.
 * See http://www.eu-egee.org/partners/ for details on the
 * copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.glite.apel.publisher;

import org.glite.apel.core.config.parser.ConfigMap;

/**
 * Represents a mapping of valid tokens the Apel XML Parser will interpret
 * to internal ids (which are used by the publisher application).
 *
 * The map is fed into the XMLParser and is used to validate the
 * users XML config file.
 */
class PublisherMap extends ConfigMap {
    /**
     * Root element of the XML configuration file.
     */
    private static final String ROOT = "/ApelConfiguration";
    /**
     * XML element defining database URL.
     */
    private static final String DBURL = "/DBURL";
    /**
     * XML element defining database username.
     */
    private static final String DBUSERNAME = "/DBUsername";
    /**
     * XML element defining database password.
     */
    private static final String DBPASSWORD = "/DBPassword";
    /**
     * XML element defining the site name.
     */
    private static final String SITENAME = "/SiteName";
     /**
      * Cristina del Cano Novales 2008-06-04
      * Bug fix #34506 Added limit to avoid OutOfMemory error
      * Number of records to publish in each batch
     */
    private static final String LIMIT = "/Limit";

    private static final String MAX_INSERT_BATCH = "/MaxInsertBatch";

    private static final String CONSUMER_REPLY_TIMEOUT = "/ConsumerTimeout";

    /**
     * XML element defining the join processor.
     */
    private static final String JOIN_PROCESSOR = "/JoinProcessor";
    /**
     * XML element defining the strategy for republishing records.
     */
    private static final String REPUBLISH = "/JoinProcessor/Republish";
    /**
     * XML element defining the strategy for republishing records starting from RecordStart
     */
    private static final String REPUBLISH_RECORDSTART = "/JoinProcessor/Republish@recordStart";
    /**
     * XML element defining the strategy for republishing records starting from RecordEnd
     */
    private static final String REPUBLISH_RECORDEND = "/JoinProcessor/Republish@recordEnd";
    /**
     * XML element defining the strategy for republishing records starting from disableJoin
     */
    private static final String REPUBLISH_NOJOIN = "/JoinProcessor@disableJoin";    
    /**
     * XML element defining whether to include global user names in output.
     */
    private static final String PUBLISH_GLOBAL_USER = "/JoinProcessor@publishGlobalUserName";
    /**
     * XML element defining the location of the public key used to encrypt UserDN
     */
    private static final String PUBLIC_KEY_LOCATION = "/JoinProcessor@publicKeyLocation";
    /**
     * XML element defining the delete processor.
     */
    private static final String DB_DELETE_PROCESSOR = "/DBDeleteProcessor";
    /**
     * XML element defining the delete processor.
     */
    private static final String CLEAN_ALL = "/DBDeleteProcessor@cleanAll";

    /**
     * XML element defining the delete processor.
     */
    private static final String DB_PROCESSOR = "/DBProcessor";
    /**
     * XML element defining the delete processor.
     */
    private static final String INSPECT_TABLE = "/DBProcessor@inspectTables";

    /**
     * XML element defining whether to enable debug logging.
     */
    private static final String DEBUG = "@enableDebugLogging";
    /**
     * XML element defining the strategy for filtering records by VO
     */
    private static final String VOINCLUDE = "/JoinProcessor/VOInclude";
    /**
     * XML element defining the type of including
     */
    private static final String VOINCLUDE_INCLUDE = "/JoinProcessor/VOInclude@include";
    /**
     * XML element defining the list of VOs to include/exclude
     */
    private static final String VOINCLUDE_VOLIST = "/JoinProcessor/VOInclude/VOList";
    /**
     * XML element list with the individual VOs to include/exclude
     */
    private static final String VOINCLUDE_VOLIST_VO = "/JoinProcessor/VOInclude/VOList/VO";
    /**
     * XML element defining the strategy for mapping between unix groups and VOs
     */
    private static final String VOINCLUDE_VOMAPPING = "/JoinProcessor/VOInclude/VOMapping";
    /**
     * XML element list with the mappings between unix groups and VOs
     */
    private static final String VOINCLUDE_VOMAPPING_VOGROUP = "/JoinProcessor/VOInclude/VOMapping/VOGroup";
    /**
     * Site name.
     */
    static final String PROP_SITE = "site";
    /**
     * GK Submit host name.
     */
    static final String PROP_SUBMIT_HOST = "CEhost";
    /**
     * Database URL.
     */
     static final String PROP_DB_URL = "DBURL";
    /**
     * Database user name.
     */
     static final String PROP_DB_USERNAME = "DBUsername";
    /**
     * Database password.
     */
     static final String PROP_DB_PASSWORD = "DBPassword";
    /**
     * Quiet mode for minimal debug logging.
     */
     static final String OPT_ENABLE_LOGGING = "logenabled";
    /**
     * Delete all records held in tables on the MON box.
     */
     static final String OPT_DB_DELETE_PROCESSOR = "d";
    /**
     * Perform the join that inserts data into LcgRecords table.
     */
     static final String OPT_JOIN_PROCESSOR = "j";
    /**
     * Internal id - option to run the database processor.
     */
     static final String OPT_DB_PROCESSOR = "p";
    /**
     * If option is set, the DN of the user will be published.
     */
     static final String OPT_PUBLISH_GLOBAL_USER = "publishGlobalDN";
    /**
     * If option is set, an external public key file is used
     */
     static final String OPT_PUBLIC_KEY_LOCATION = "publicKeyLocation";
    /**
     * Option set when using the DeleteLogProcessor.
     */
     static final String OPT_CLEAN_ALL_FROM_TABLES = "cleanAll";
    /**
     * Option set when using the DBProcessor.
     */
     static final String OPT_INSPECT_TABLES = "inspectTables";
    /**
     * Republish mode used to recover lost accounting records.
     */
     static final String PROP_REPUBLISH = "republish";
    /**
     * Republish mode start interval for publish.
     */
     static final String OPT_REPUBLISH_RECORDSTART = "recordStart";
    /**
     * Republish mode end interval for publish.
     */
     static final String OPT_REPUBLISH_RECORDEND = "recordEnd";
    /**
     * Republish mode, diable join option
     */
     static final String OPT_REPUBLISH_NOJOIN = "disableJoin";
     /**
      * VOInclude mode, include option
      */
     static final String OPT_VOINCLUDE_INCLUDE = "include";
    /**
     * Republishing missing records (used in connection with REPUBLISH
     * property).
     */
     static final String REPUBLISH_MISSING = "missing";
    /**
     * Republishing all records (used in connection with REPUBLISH property).
     */
     static final String REPUBLISH_ALL = "all";
    /**
     * DaveKant
     * Republishing all records in a defined gap
     */
     static final String REPUBLISH_GAP = "gap";
    /**
     * Republishing no records (used in connection with REPUBLISH property).
     */
     static final String REPUBLISH_NOTHING = "nothing";
 

     /**
      * The max. num of accounting records in each JMS message
      */
     static final String PROP_MAX_INSERT_BATCH = "The max. num of accounting records in each JMS message";

     static final String PROP_CONSUMER_REPLY_TIMEOUT = "Timeout for awaiting" +
             " a reply from the consumer.";

     static final String PROP_VO = "VOList";
     static final String PROP_VOGROUP = "VOGroup";

     static final String PROP_LIMIT = "Batch size";

         PublisherMap() {
          setupOptions();
          setupParameters();
          setupTokens();
         }
        /**
         * Adds the valid tokens that make up the config syntax. If an XML
         * element is parsed and does not exist within the list of known tokens,         * an error will be flagged.
         */
         private void setupTokens() {
          addTok(ROOT);
          addTok(ROOT + DBURL);
          addTok(ROOT + DBUSERNAME);
          addTok(ROOT + DBPASSWORD);
          addTok(ROOT + SITENAME);
          addTok(ROOT + LIMIT);
          addTok(ROOT + MAX_INSERT_BATCH);
          addTok(ROOT + CONSUMER_REPLY_TIMEOUT);
          addTok(ROOT + CLEAN_ALL);
          addTok(ROOT + JOIN_PROCESSOR);
          addTok(ROOT + DB_DELETE_PROCESSOR);
          addTok(ROOT + DB_PROCESSOR);
          addTok(ROOT + REPUBLISH);
          addTok(ROOT + VOINCLUDE);
          addTok(ROOT + VOINCLUDE_VOLIST);
          addTok(ROOT + VOINCLUDE_VOLIST_VO);
          addTok(ROOT + VOINCLUDE_VOMAPPING);
          addTok(ROOT + VOINCLUDE_VOMAPPING_VOGROUP);
         }
        /**
         * Adds all the user options to internal id mappings.
         */
         private void setupOptions() {
          addOpt(ROOT + DEBUG, OPT_ENABLE_LOGGING);
          addOpt(ROOT + DB_DELETE_PROCESSOR, OPT_DB_DELETE_PROCESSOR);
          addOpt(ROOT + DB_PROCESSOR, OPT_DB_PROCESSOR);
          addOpt(ROOT + JOIN_PROCESSOR, OPT_JOIN_PROCESSOR);
          addOpt(ROOT + PUBLISH_GLOBAL_USER, OPT_PUBLISH_GLOBAL_USER);
          addOpt(ROOT + PUBLIC_KEY_LOCATION, OPT_PUBLIC_KEY_LOCATION);
          addOpt(ROOT + REPUBLISH_RECORDSTART, OPT_REPUBLISH_RECORDSTART);
          addOpt(ROOT + REPUBLISH_RECORDEND, OPT_REPUBLISH_RECORDEND);
          addOpt(ROOT + REPUBLISH_NOJOIN, OPT_REPUBLISH_NOJOIN);
          addOpt(ROOT + CLEAN_ALL, OPT_CLEAN_ALL_FROM_TABLES);
          addOpt(ROOT + INSPECT_TABLE, OPT_INSPECT_TABLES);
          addOpt(ROOT + VOINCLUDE_INCLUDE, OPT_VOINCLUDE_INCLUDE);

         }
        /**
         * Adds all the user parameters to internal id mappings.
         */
         private void setupParameters() {
          addParam(ROOT + DBURL, PROP_DB_URL);
          addParam(ROOT + DBUSERNAME, PROP_DB_USERNAME);
          addParam(ROOT + DBPASSWORD, PROP_DB_PASSWORD);
          addParam(ROOT + SITENAME, PROP_SITE);
          addParam(ROOT + LIMIT, PROP_LIMIT);
          addParam(ROOT + MAX_INSERT_BATCH, PROP_MAX_INSERT_BATCH);
          addParam(ROOT + CONSUMER_REPLY_TIMEOUT, PROP_CONSUMER_REPLY_TIMEOUT);
          addParam(ROOT + REPUBLISH, PROP_REPUBLISH);
          addParam(ROOT + VOINCLUDE_VOLIST_VO, PROP_VO);
          addParam(ROOT + VOINCLUDE_VOMAPPING_VOGROUP, PROP_VOGROUP);
         }
}
