/*
 * Copyright (c) Members of the EGEE Collaboration. 2004-2010.
 * See http://www.eu-egee.org/partners/ for details on the
 * copyright holders. 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.glite.apel.publisher;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;

import org.edg.info.LatestProducer;
import org.edg.info.RGMAException;
import org.glite.apel.core.ApelException;
import org.glite.apel.core.db.ApelDB;

import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * Unit tests to validate the functionality of the LcgRecordsConsumer.
 */
public class AccountCheckerTest extends TestCase {

    private Connection m_connection;
    
    private AccountChecker m_accountChecker;

    private String m_siteName;
    
    private LatestProducer m_lp;
    
    private String m_date;
    
    private String m_time;
    
    public static junit.framework.Test suite() {
        return new TestSuite(AccountCheckerTest.class);
    }
    
    public void setUp() {
        String db = "jdbc:mysql://localhost/test";
        String user = "test";
        String pass = "info";
        String driver = "org.gjt.mm.mysql.Driver";
        m_siteName = "Example-Test";
        try {
            Class.forName(driver);
            m_connection = DriverManager.getConnection(db, user, pass);
        } catch (Exception e) {
            e.printStackTrace();
            fail();
        }
      	scrubRegistry();
        try {
        	m_accountChecker = new AccountChecker();
        } catch (ApelException e) {
        	fail();
        }
        setupDateTime();
    }

	public void tearDown() {
        scrubRegistry();
        deleteRecords();
        if (m_lp != null) {
        	m_lp.close();
        }
        close(m_connection);
    }
    
    /**
     * Closes an sql Connection, catching any exceptions that are thrown.
     */
    void close(Connection conn) {
        try {
            conn.close();
        } catch (Exception e) {}
    }

    /**
     * Closes an sql Statement, catching any exceptions that are thrown.
     */
    void close(Statement stmt) {
        try {
            stmt.close();
        } catch (Exception e) {}
    }
    
    /**
     * Scrubs all the tables used by the Apel program.
     */
    private void deleteRecords() {
        Statement stmt = null;
        try {
            stmt = m_connection.createStatement();
            stmt.executeUpdate("DELETE FROM " + ApelDB.REPUBLISH_TABLE_NAME);
            stmt.executeUpdate("DELETE FROM " + ApelDB.MSG_TABLE_NAME);
            stmt.executeUpdate("DELETE FROM " + ApelDB.GK_TABLE_NAME);
            stmt.executeUpdate("DELETE FROM " + ApelDB.SPEC_TABLE_NAME);
            stmt.executeUpdate("DELETE FROM " + ApelDB.EVENT_TABLE_NAME);
            stmt.executeUpdate("DELETE FROM " + ApelDB.LCG_RECORDS_TABLE_NAME);
            stmt.executeUpdate("DELETE FROM " + ApelDB.PROCESSED_FILES_TABLE_NAME);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            close(stmt);
        }
    }

    /**
     * Sets-up date/time.
     */
    private void setupDateTime() {
        SimpleDateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd");
        dateFormatter.setTimeZone(TimeZone.getTimeZone("UTC"));
        SimpleDateFormat timeFormatter = new SimpleDateFormat("HH:mm:ss");
        timeFormatter.setTimeZone(TimeZone.getTimeZone("UTC"));
        Date currentTime = Calendar.getInstance().getTime();
        m_date = dateFormatter.format(currentTime);
        m_time = timeFormatter.format(currentTime);
    }
    
    /**
     * Tests if no archiver is present, -1 is returned.
     */
    public void testEmptyArchiver() {
        try {
            int count = m_accountChecker.countArchivedRecords(m_date, 
                m_time, m_siteName);
            assertEquals(count, -1);
        } catch (ApelException e) {
        	e.printStackTrace();
            fail();
        }
    }

    /**
     * Tests the count operation returns an the exact number of tuples
     * from the specified date. 
     */
    public void testCountArchivedRecords() {
        try {
        	insertJunk(m_date, m_time);
            int count = m_accountChecker.countArchivedRecords(m_date, 
                m_time, m_siteName);
            assertEquals(count, 1);
            setupDateTime();
            count = m_accountChecker.countArchivedRecords(m_date, 
                m_time, m_siteName);
            assertEquals(count, 0);
        } catch (Exception e) {
        	e.printStackTrace();
            fail();
        }
    }
    
    private void insertJunk(String date, String time) throws RGMAException {
    	m_lp = new LatestProducer();
        m_lp.declareTable(ApelDB.LCG_RECORDS_TABLE_NAME, "");
        m_lp.insert("INSERT INTO " + ApelDB.LCG_RECORDS_TABLE_NAME + 
            " VALUES ('TEST','" + m_siteName + "','','','','','','','', " +
            "0, 0, '', '', '', '', 0, 0, '', 0, 0, 0, 0, '', '', '" + date + 
            "', '" + time + "')");
    }
    
    private void scrubRegistry() {
        Statement stmt = null;
        Connection con = null;
        try {
            con = DriverManager.getConnection("jdbc:mysql://localhost/registry", 
                "registry", "info");
            stmt = con.createStatement();
            stmt.executeUpdate("DELETE FROM consumers");
            stmt.executeUpdate("DELETE FROM producers");
        } catch (SQLException e) {
        	e.printStackTrace();
            fail();
        }
        finally {
            close(stmt);
            close(con);
        }
    }
    
}
