/*
 * Copyright (c) Members of the EGEE Collaboration. 2004-2010.
 * See http://www.eu-egee.org/partners/ for details on the
 * copyright holders. 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.glite.apel.publisher;

import java.io.FileNotFoundException;
import java.io.StringReader;

import org.glite.apel.core.config.parser.Config;
import org.glite.apel.core.config.parser.ConfigurationException;
import org.glite.apel.core.config.parser.XMLParser;

import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * Unit tests to verify the XML parser using the Apel config file 
 * format.
 */
public class ParserTest extends TestCase {

	private static String m_configFileStart = 
    	"<ApelConfiguration>";
    private static String m_configFileEnd = 
    	"</ApelConfiguration>";
    private static String m_yesValue = "yes";
    private static String m_noValue = "no";

    public static junit.framework.Test suite() {
        return new TestSuite(ParserTest.class);
    }

    public void setUp() {}
    
    /**
     * Tests if an invalid file is given, a FileNotFoundException 
     * is thrown.
     */
    public void testFileNotFound() {
        try {
            new XMLParser("/a/file/that/doesnt/exist");
            fail("invalid file does not through exception");
        } catch (FileNotFoundException e) {}
    }
    
    /**
     * Tests a ConfigurationException is thrown when parsing
     * badly formed XML.
     */
    public void testFileNotWellFormed() {
        try {
            XMLParser source = new XMLParser(new StringReader(m_configFileStart));
            source.parse(new PublisherMap());
            fail("parser did not throw exception for badly formed xml");
        } catch (ConfigurationException e) {}
    }    

    /**
     * Tests an exception is thrown for XML which is not part of the
     * standard Apel config syntax.
     */
    public void testInvalidConfig() {
        try {
            String xml = m_configFileStart + "<funky-monkey>" +
				"</funky-monkey>" + m_configFileEnd;
            XMLParser parser = new XMLParser(new StringReader(xml));
            parser.parse(new PublisherMap());
            fail("invalid xml element should thrown an exception");
        } catch (ConfigurationException e) {}
    }
    
    
    /**
     * Tests the enable debug logging flag is set when the related XML 
     * attribute is parsed.
     */
    public void testEnableDebugLogging() {
        try {
            String xml = "<ApelConfiguration" + 
				" enableDebugLogging=\"" + m_noValue + "\"/>";
            XMLParser parser = new XMLParser(new StringReader(xml));
            Config config = parser.parse(new PublisherMap());
            assertFalse(config.isOptionSet(PublisherMap.OPT_ENABLE_LOGGING));
        } catch (ConfigurationException e) {
            fail(e.toString());
        }    	
    }
    
    
    /**
     * Tests the print global user name option is set correctly when the join 
     * processor is parsed.
     */
    public void testPrintGlobalUserName() {
        try {
            String xml = m_configFileStart + "<JoinProcessor " +
                "publishGlobalUserName=\"" + m_yesValue + "\"/>" + 
                m_configFileEnd;
            XMLParser parser = new XMLParser(new StringReader(xml));
            Config config = parser.parse(new PublisherMap());
            assertTrue(config.isOptionSet(PublisherMap.OPT_PUBLISH_GLOBAL_USER));
        } catch (ConfigurationException e) {
            fail(e.toString());
        }
    }

    /**
     * Tests the join processor is set when the related XML 
     * element is parsed.
     */
    public void testJoinProcessing() {
        try {
            String xml = m_configFileStart + "<JoinProcessor/>" + m_configFileEnd;
            XMLParser parser = new XMLParser(new StringReader(xml));
            Config config = parser.parse(new PublisherMap());
            assertTrue(config.isOptionSet(PublisherMap.OPT_JOIN_PROCESSOR));
        } catch (ConfigurationException e) {
            fail(e.toString());
        }
    }

    /**
     * Tests the republish property is correctly set.
     */
    public void testRepublishAll() {
        try {
            String republishMode = "all";
            String xml = m_configFileStart + "<JoinProcessor>" + 
                "<Republish>" + republishMode + "</Republish>" + 
                "</JoinProcessor>" + m_configFileEnd;
            XMLParser parser = new XMLParser(new StringReader(xml));
            Config config = parser.parse(new PublisherMap());
            assertTrue(config.isPropertySet(PublisherMap.PROP_REPUBLISH));
            String prop = config.getProperty(PublisherMap.PROP_REPUBLISH);
            assertEquals(prop, republishMode);
        } catch (ConfigurationException e) {
            fail(e.toString());
        }
    }
    
    /**
     * Tests the republish property is correctly set.
     */
    public void testRepublishMissing() {
        try {
            String republishMode = "missing";
            String xml = m_configFileStart + "<JoinProcessor>" + 
                "<Republish>" + republishMode + "</Republish>" + 
                "</JoinProcessor>" + m_configFileEnd;
            XMLParser parser = new XMLParser(new StringReader(xml));
            Config config = parser.parse(new PublisherMap());
            assertTrue(config.isPropertySet(PublisherMap.PROP_REPUBLISH));
            String prop = config.getProperty(PublisherMap.PROP_REPUBLISH);
            assertEquals(prop, republishMode);
        } catch (ConfigurationException e) {
            fail(e.toString());
        }
    }    
    
}
